#ifndef _TestCullingTerrainApplication_H__
#define _TestCullingTerrainApplication_H__

#include "CEGUIForwardRefs.h"
#include <OgreRenderTargetListener.h>
#include "ExampleApplication.h"

#include "OgreTerrainContentGenerator.h"
#include "TerrainFrameListener.h"

class VisualizationRenderTargetListener: public RenderTargetListener
{
public:
	VisualizationRenderTargetListener(SceneManager *sceneMgr);

protected:
	void preViewportUpdate (const RenderTargetViewportEvent &evt);
	void postRenderTargetUpdate (const RenderTargetEvent &evt);

	SceneManager *mSceneMgr;
	
	ShadowTechnique mSavedShadowTechnique;
	ColourValue mSavedAmbientLight;
};


class TestCullingTerrainApplication: public ExampleApplication
{
public:
	TestCullingTerrainApplication();
	~TestCullingTerrainApplication();

	void generateScene(int num, int objectType);

	String getCurrentObjectCaption(int id);

	std::vector<AnimationState *> &getAnimationStates();
	std::vector<Real> &getAnimationSpeed();

protected:

	//-- inherited from ExampleApplication
	bool setup();
	void createScene();
	void createFrameListener();
	void createCamera();
	//void createViewports();
	
	virtual void createRenderTargetListener();

	std::vector<AnimationState *> mAnimationStates;
	std::vector<Real> mAnimationSpeed;

	/** cegui setup */
	void setupGui();

	CEGUI::OgreCEGUIRenderer *mGUIRenderer;
	CEGUI::System *mGUISystem;

	Vector3 mMinTranslation;
	Vector3 mMaxTranslation;

	Vector3 mMinAngle;
	Vector3 mMaxAngle;

	TerrainContentGenerator *mTerrainContentGenerator;
	
	Camera *mVizCamera;
	SceneNode *mCamNode;
	Light *mSunLight;

	TerrainFrameListener *mTerrainFrameListener;
	ColourValue mAmbientLight;
	
private:
	void chooseSceneManager(void);
};

#endif // TestCullingTerrainApplication